@@ -1,6 +1,6 @@
 /*
 	acr38cmd.c: Emulated CCID reader commands for ACR38 non-CCID reader
-	Copyright (C) 2011-2013   Advanced Card Systems Ltd.
+	Copyright (C) 2011-2014   Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,21 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#include "config.h"
+#include <config.h>
+
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <pcsclite.h>
 #include <ifdhandler.h>
 #include <reader.h>
@@ -32,7 +43,6 @@
 #include "defs.h"
 #include "ccid_ifdhandler.h"
 #include "debug.h"
-#include "ccid_usb.h"
 
 #define ACR38_HEADER_SIZE		4
 #define ACR38_STATUS_OFFSET		1
@@ -81,11 +91,6 @@
 #define ACR38_OPTION_EMV_MODE		0x10	// EMV mode
 #define ACR38_OPTION_MEMCARD_MODE	0x20	// Memory card mode
 
-// Fix problem using pcsc-lite 1.6.x header files
-#ifndef IFD_ERROR_INSUFFICIENT_BUFFER
-#define IFD_ERROR_INSUFFICIENT_BUFFER 618
-#endif
-
 static RESPONSECODE ACR38_CmdXfrBlockTPDU_T0(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
 	unsigned char rx_buffer[]);
@@ -582,7 +587,7 @@ RESPONSECODE ACR38_SetCardType(unsigned int reader_index,
 				break;
 
 			default:
-				DEBUG_CRITICAL2("Card type %d is not supported", cardType);
+				DEBUG_CRITICAL2("Card type " DWORD_D " is not supported", cardType);
 				ccid_descriptor->cardType = ACR38_CARD_TYPE_MCU_AUTO;
 				return_value = IFD_COMMUNICATION_ERROR;
 				break;
@@ -604,7 +609,6 @@ RESPONSECODE ACR38_GetFirmwareVersion(unsigned int reader_index,
 	unsigned int length;
 	RESPONSECODE return_value = IFD_SUCCESS;
 	unsigned char buffer_tmp[20];
-	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	// GET_ACR_STAT
 	cmd[0] = 0x01;
