@@ -1,7 +1,7 @@
 /*
     ccid.c: CCID common code
-    Copyright (C) 2003-2005   Ludovic Rousseau
-    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
+    Copyright (C) 2003-2010   Ludovic Rousseau
+    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -19,23 +19,38 @@
 */
 
 /*
- * $Id: ccid.c 4346 2009-07-28 13:39:37Z rousseau $
+ * $Id: ccid.c 6976 2014-09-04 11:35:46Z rousseau $
  */
 
+#include <config.h>
+
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <pcsclite.h>
 #include <ifdhandler.h>
 
-#include "config.h"
 #include "debug.h"
 #include "ccid.h"
 #include "defs.h"
 #include "ccid_ifdhandler.h"
 #include "commands.h"
+#include "utils.h"
 #include "acr38cmd.h"
-#include "ccid_usb.h"
+
+#ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 
 static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pFirmwareVersion);
 static int ACR83_DisplayLcdMessage(unsigned int reader_index, const char *message);
@@ -49,17 +64,10 @@ static int ACR1222_GetFirmwareVersion(unsigned int reader_index, char *firmwareV
 int ccid_open_hack_pre(unsigned int reader_index)
 {
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
-	int doInterruptRead = 0;	// Disable InterruptRead
 	int i;
 
 	switch (ccid_descriptor->readerID)
 	{
-		case CARDMAN3121+1:
-			/* Reader announces APDU but is in fact TPDU */
-			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
-			ccid_descriptor->dwFeatures |= CCID_CLASS_TPDU;
-			break;
-
 		case MYSMARTPAD:
 			ccid_descriptor->dwMaxIFSD = 254;
 			break;
@@ -67,12 +75,24 @@ int ccid_open_hack_pre(unsigned int reader_index)
 		case CL1356D:
 			/* the firmware needs some time to initialize */
 			(void)sleep(1);
-			ccid_descriptor->readTimeout = 60; /* 60 seconds */
+			ccid_descriptor->readTimeout = 60*1000; /* 60 seconds */
 			break;
 
-		case KOBIL_TRIBANK:
-			/* the InterruptRead does not timeout (on Mac OS X) */
-			doInterruptRead = 0;
+		case GEMPCTWIN:
+		case GEMPCKEY:
+		case DELLSCRK:
+			/* Only the chipset with firmware version 2.00 is "bogus"
+			 * The reader may send packets of 0 bytes when the reader is
+			 * connected to a USB 3 port */
+			if (0x0200 == ccid_descriptor->IFD_bcdDevice)
+			{
+				ccid_descriptor->zlp = TRUE;
+				DEBUG_INFO1("ZLP fixup");
+			}
+			break;
+
+		case OZ776_7772:
+			ccid_descriptor->dwMaxDataRate = 9600;
 			break;
 
 		case ACS_ACR122U:
@@ -93,17 +113,21 @@ int ccid_open_hack_pre(unsigned int reader_index)
 			break;
 	}
 
+#ifndef __APPLE__
 	/* CCID */
-	if (doInterruptRead && (0 == ccid_descriptor->bInterfaceProtocol))
+	if (((PROTOCOL_CCID == ccid_descriptor->bInterfaceProtocol)
+		|| (PROTOCOL_ACR38 == ccid_descriptor->bInterfaceProtocol))
+		&& (3 == ccid_descriptor -> bNumEndpoints))
 	{
 #ifndef TWIN_SERIAL
-		/* just wait for 10ms in case a notification is in the pipe */
-		(void)InterruptRead(reader_index, 10);
+		/* just wait for 100ms in case a notification is in the pipe */
+		(void)InterruptRead(reader_index, 100);
 #endif
 	}
+#endif
 
 	/* ICCD type A */
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char tmp[MAX_ATR_SIZE];
 		unsigned int n = sizeof(tmp);
@@ -115,7 +139,7 @@ int ccid_open_hack_pre(unsigned int reader_index)
 	}
 
 	/* ICCD type B */
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char tmp[MAX_ATR_SIZE];
 		unsigned int n = sizeof(tmp);
@@ -137,6 +161,102 @@ int ccid_open_hack_pre(unsigned int reader_index)
 	return 0;
 } /* ccid_open_hack_pre */
 
+#ifndef NO_LOG
+/*****************************************************************************
+ *
+ *					dump_gemalto_firmware_features
+ *
+ ****************************************************************************/
+static void dump_gemalto_firmware_features(struct GEMALTO_FIRMWARE_FEATURES *gff)
+{
+	DEBUG_INFO2("Dumping Gemalto firmware features (%zd bytes):",
+		sizeof(struct GEMALTO_FIRMWARE_FEATURES));
+
+#define YESNO(x) (x) ? "yes" : "no"
+
+	DEBUG_INFO2(" bLogicalLCDLineNumber: %d", gff->bLogicalLCDLineNumber);
+	DEBUG_INFO2(" bLogicalLCDRowNumber: %d", gff->bLogicalLCDRowNumber);
+	DEBUG_INFO2(" bLcdInfo: 0x%02X", gff->bLcdInfo);
+	DEBUG_INFO2(" bEntryValidationCondition: 0x%02X",
+		gff->bEntryValidationCondition);
+
+	DEBUG_INFO1(" Reader supports PC/SCv2 features:");
+	DEBUG_INFO2("  VerifyPinStart: %s", YESNO(gff->VerifyPinStart));
+	DEBUG_INFO2("  VerifyPinFinish: %s", YESNO(gff->VerifyPinFinish));
+	DEBUG_INFO2("  ModifyPinStart: %s", YESNO(gff->ModifyPinStart));
+	DEBUG_INFO2("  ModifyPinFinish: %s", YESNO(gff->ModifyPinFinish));
+	DEBUG_INFO2("  GetKeyPressed: %s", YESNO(gff->GetKeyPressed));
+	DEBUG_INFO2("  VerifyPinDirect: %s", YESNO(gff->VerifyPinDirect));
+	DEBUG_INFO2("  ModifyPinDirect: %s", YESNO(gff->ModifyPinDirect));
+	DEBUG_INFO2("  Abort: %s", YESNO(gff->Abort));
+	DEBUG_INFO2("  GetKey: %s", YESNO(gff->GetKey));
+	DEBUG_INFO2("  WriteDisplay: %s", YESNO(gff->WriteDisplay));
+	DEBUG_INFO2("  SetSpeMessage: %s", YESNO(gff->SetSpeMessage));
+	DEBUG_INFO2("  bTimeOut2: %s", YESNO(gff->bTimeOut2));
+	DEBUG_INFO2("  bPPDUSupportOverXferBlock: %s",
+		YESNO(gff->bPPDUSupportOverXferBlock));
+	DEBUG_INFO2("  bPPDUSupportOverEscape: %s",
+		YESNO(gff->bPPDUSupportOverEscape));
+
+	DEBUG_INFO2(" bListSupportedLanguages: %s",
+		YESNO(gff->bListSupportedLanguages));
+	DEBUG_INFO2(" bNumberMessageFix: %s", YESNO(gff->bNumberMessageFix));
+
+	DEBUG_INFO2(" VersionNumber: 0x%02X", gff->VersionNumber);
+	DEBUG_INFO2(" MinimumPINSize: %d", gff->MinimumPINSize);
+	DEBUG_INFO2(" MaximumPINSize: %d", gff->MaximumPINSize);
+	DEBUG_INFO2(" Firewall: %s", YESNO(gff->Firewall));
+	if (gff->Firewall && gff->FirewalledCommand_SW1
+		&& gff->FirewalledCommand_SW2)
+	{
+		DEBUG_INFO2("  FirewalledCommand_SW1: 0x%02X",
+			gff->FirewalledCommand_SW1);
+		DEBUG_INFO2("  FirewalledCommand_SW2: 0x%02X",
+			gff->FirewalledCommand_SW2);
+	}
+
+} /* dump_gemalto_firmware_features */
+#endif
+
+/*****************************************************************************
+ *
+ *					set_gemalto_firmware_features
+ *
+ ****************************************************************************/
+static void set_gemalto_firmware_features(unsigned int reader_index)
+{
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	struct GEMALTO_FIRMWARE_FEATURES *gf_features;
+
+	gf_features = malloc(sizeof(struct GEMALTO_FIRMWARE_FEATURES));
+	if (gf_features)
+	{
+		unsigned char cmd[] = { 0x6A }; /* GET_FIRMWARE_FEATURES command id */
+		unsigned int len_features = sizeof *gf_features;
+		RESPONSECODE ret;
+
+		ret = CmdEscape(reader_index, cmd, sizeof cmd,
+			(unsigned char*)gf_features, &len_features, 0);
+		if ((IFD_SUCCESS == ret) &&
+		    (len_features == sizeof *gf_features))
+		{
+			/* Command is supported if it succeeds at CCID level */
+			/* and returned size matches our expectation */
+			ccid_descriptor->gemalto_firmware_features = gf_features;
+#ifndef NO_LOG
+			dump_gemalto_firmware_features(gf_features);
+#endif
+		}
+		else
+		{
+			/* Command is not supported, let's free allocated memory */
+			free(gf_features);
+			DEBUG_INFO3("GET_FIRMWARE_FEATURES failed: " DWORD_D ", len=%d",
+				ret, len_features);
+		}
+	}
+} /* set_gemalto_firmware_features */
+
 /*****************************************************************************
  *
  *					ccid_open_hack_post
@@ -145,19 +265,20 @@ int ccid_open_hack_pre(unsigned int reader_index)
 int ccid_open_hack_post(unsigned int reader_index)
 {
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	RESPONSECODE return_value = IFD_SUCCESS;
 
 	switch (ccid_descriptor->readerID)
 	{
 		case GEMPCKEY:
 		case GEMPCTWIN:
-			/* Reader announces TPDU but can do APDU */
+			/* Reader announces TPDU but can do APDU (EMV in fact) */
 			if (DriverOptions & DRIVER_OPTION_GEMPC_TWIN_KEY_APDU)
 			{
-				unsigned char cmd[] = { 0xA0, 0x02 };
+				unsigned char cmd[] = { 0x1F, 0x02 };
 				unsigned char res[10];
 				unsigned int length_res = sizeof(res);
 
-				if (CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res) == IFD_SUCCESS)
+				if (CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res, 0) == IFD_SUCCESS)
 				{
 					ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
 					ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
@@ -165,6 +286,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 			}
 			break;
 
+		case VEGAALPHA:
 		case GEMPCPINPAD:
 			/* load the l10n strings in the pinpad memory */
 			{
@@ -273,10 +395,28 @@ int ccid_open_hack_post(unsigned int reader_index)
 					"Card Error",
 					"PIN blocked" };
 
-				char *lang;
+				const char *lang;
 				const char **l10n;
 
+#ifdef __APPLE__
+				CFArrayRef cfa;
+				CFStringRef slang;
+
+				/* Get the complete ordered list */
+				cfa = CFLocaleCopyPreferredLanguages();
+
+				/* Use the first/preferred language
+				 * As the driver is run as root we get the language
+				 * selected during install */
+				slang = CFArrayGetValueAtIndex(cfa, 0);
+
+				/* CFString -> C string */
+				lang = CFStringGetCStringPtr(slang, kCFStringEncodingMacRoman);
+#else
+				/* The other Unixes just use the LANG env variable */
 				lang = getenv("LANG");
+#endif
+				DEBUG_COMM2("Using lang: %s", lang);
 				if (NULL == lang)
 					l10n = en;
 				else
@@ -299,6 +439,10 @@ int ccid_open_hack_post(unsigned int reader_index)
 						l10n = en;
 				}
 
+#ifdef __APPLE__
+				/* Release the allocated array */
+				CFRelease(cfa);
+#endif
 				offset = 0;
 				cmd[offset++] = 0xB2;	/* load strings */
 				cmd[offset++] = 0xA0;	/* address of the memory */
@@ -319,17 +463,42 @@ int ccid_open_hack_post(unsigned int reader_index)
 				}
 
 				(void)sleep(1);
-				if (IFD_SUCCESS == CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res))
+				if (IFD_SUCCESS == CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res, DEFAULT_COM_READ_TIMEOUT))
 				{
 					DEBUG_COMM("l10n string loaded successfully");
 				}
 				else
 				{
 					DEBUG_COMM("Failed to load l10n strings");
+					return_value = IFD_COMMUNICATION_ERROR;
+				}
+
+				if (DriverOptions & DRIVER_OPTION_DISABLE_PIN_RETRIES)
+				{
+					/* disable VERIFY from reader */
+					const unsigned char cmd2[] = {0xb5, 0x00};
+					length_res = sizeof(res);
+					if (IFD_SUCCESS == CmdEscape(reader_index, cmd2, sizeof(cmd2), res, &length_res, DEFAULT_COM_READ_TIMEOUT))
+					{
+						DEBUG_COMM("Disable SPE retry counter successfull");
+					}
+					else
+					{
+						DEBUG_CRITICAL("Failed to disable SPE retry counter");
+					}
 				}
 			}
 			break;
 
+		case HPSMARTCARDKEYBOARD:
+		case HP_CCIDSMARTCARDKEYBOARD:
+		case FUJITSUSMARTKEYB:
+			/* the Secure Pin Entry is bogus so disable it
+			 * http://martinpaljak.net/2011/03/19/insecure-hp-usb-smart-card-keyboard/
+			 */
+			ccid_descriptor->bPINSupport = 0;
+			break;
+
 #if 0
 		/* SCM SCR331-DI contactless */
 		case SCR331DI:
@@ -350,8 +519,8 @@ int ccid_open_hack_post(unsigned int reader_index)
 				unsigned char res[20];
 				unsigned int length_res = sizeof(res);
 
-				if ((IFD_SUCCESS == CmdEscape(reader_index, cmd1, sizeof(cmd1), res, &length_res))
-					&& (IFD_SUCCESS == CmdEscape(reader_index, cmd2, sizeof(cmd2), res, &length_res)))
+				if ((IFD_SUCCESS == CmdEscape(reader_index, cmd1, sizeof(cmd1), res, &length_res, 0))
+					&& (IFD_SUCCESS == CmdEscape(reader_index, cmd2, sizeof(cmd2), res, &length_res, 0)))
 				{
 					DEBUG_COMM("SCM SCR331-DI contactless detected");
 				}
@@ -384,7 +553,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 
 				ccid_descriptor->wLcdLayout = 0x0210;
 
-				DEBUG_INFO("Getting ACR83U firmware version...");
+				DEBUG_INFO1("Getting ACR83U firmware version...");
 				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion))
 				{
 					DEBUG_INFO2("ACR83U firmware version: 0x%04X", firmwareVersion);
@@ -423,7 +592,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 				ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
 				ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
 
-				DEBUG_INFO("Getting ACR85 ICC firmware version...");
+				DEBUG_INFO1("Getting ACR85 ICC firmware version...");
 				if (ACR83_GetFirmwareVersion(reader_index, &firmwareVersion))
 				{
 					DEBUG_INFO2("ACR85 ICC firmware version: 0x%04X", firmwareVersion);
@@ -446,7 +615,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 				char firmwareVersion[30];
 				unsigned int firmwareVersionLen = sizeof(firmwareVersion);
 
-				DEBUG_INFO("Getting ACR1222 firmware version...");
+				DEBUG_INFO1("Getting ACR1222 firmware version...");
 				if (ACR1222_GetFirmwareVersion(reader_index, firmwareVersion, &firmwareVersionLen))
 				{
 					DEBUG_INFO2("ACR1222 firmware version: %s", firmwareVersion);
@@ -457,7 +626,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 					if ((ccid_descriptor->firmwareFixEnabled) &&
 						(ccid_descriptor->bCurrentSlotIndex == 1))
 					{
-						DEBUG_INFO("Enabling PICC...");
+						DEBUG_INFO1("Enabling PICC...");
 						EnablePicc(reader_index, 1);
 					}
 				}
@@ -477,7 +646,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 				char firmwareVersion[30];
 				unsigned int firmwareVersionLen = sizeof(firmwareVersion);
 
-				DEBUG_INFO("Getting ACR85 PICC firmware version...");
+				DEBUG_INFO1("Getting ACR85 PICC firmware version...");
 				if (ACR1222_GetFirmwareVersion(reader_index, firmwareVersion, &firmwareVersionLen))
 				{
 					DEBUG_INFO2("ACR85 PICC firmware version: %s", firmwareVersion);
@@ -487,7 +656,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 
 					if (ccid_descriptor->firmwareFixEnabled)
 					{
-						DEBUG_INFO("Enabling PICC...");
+						DEBUG_INFO1("Enabling PICC...");
 						EnablePicc(reader_index, 1);
 					}
 				}
@@ -594,6 +763,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 			{
 				ccid_descriptor->dwFeatures = 0x0004047A;	// Contactless
 				ccid_descriptor->dwMaxDataRate = 344100;
+				ccid_descriptor->isSamSlot = 0;
 			}
 			else
 			{
@@ -642,7 +812,11 @@ int ccid_open_hack_post(unsigned int reader_index)
 			break;
 	}
 
-	return 0;
+	/* Gemalto readers may report additional information */
+	if (GET_VENDOR(ccid_descriptor->readerID) == VENDOR_GEMALTO)
+		set_gemalto_firmware_features(reader_index);
+
+	return return_value;
 } /* ccid_open_hack_post */
 
 /*****************************************************************************
@@ -652,6 +826,7 @@ int ccid_open_hack_post(unsigned int reader_index)
  ****************************************************************************/
 void ccid_error(int error, const char *file, int line, const char *function)
 {
+#ifndef NO_LOG
 	const char *text;
 	char var_text[30];
 
@@ -769,11 +944,13 @@ void ccid_error(int error, const char *file, int line, const char *function)
 			break;
 	}
 	log_msg(PCSC_LOG_ERROR, "%s:%d:%s %s", file, line, function, text);
+#endif
 
 } /* ccid_error */
 
 void acr38_error(int error, const char *file, int line, const char *function)
 {
+#ifndef NO_LOG
 	const char *text;
 	char var_text[30];
 
@@ -835,6 +1012,7 @@ void acr38_error(int error, const char *file, int line, const char *function)
 	}
 
 	log_msg(PCSC_LOG_ERROR, "%s:%d:%s %s", file, line, function, text);
+#endif
 }
 
 // Enable PICC
@@ -847,13 +1025,13 @@ void EnablePicc(unsigned int reader_index, int enabled)
 	unsigned char antennaOn[]  = { 0xFF, 0x00, 0x00, 0x00, 0x04, 0xD4, 0x32, 0x01, 0x03 };
 
 	unsigned char response[300];
-	int responseLen;
+	unsigned int responseLen;
 
 	if (enabled)
 	{
 		// Turn ON polling
 		responseLen = sizeof(response);
-		if (CmdEscape(reader_index, pollingOn, sizeof(pollingOn), response, &responseLen) != IFD_SUCCESS)
+		if (CmdEscape(reader_index, pollingOn, sizeof(pollingOn), response, &responseLen, 0) != IFD_SUCCESS)
 		{
 			DEBUG_CRITICAL("Polling ON failed");
 		}
@@ -869,7 +1047,7 @@ void EnablePicc(unsigned int reader_index, int enabled)
 	{
 		// Turn OFF polling
 		responseLen = sizeof(response);
-		if (CmdEscape(reader_index, pollingOff, sizeof(pollingOff), response, &responseLen) != IFD_SUCCESS)
+		if (CmdEscape(reader_index, pollingOff, sizeof(pollingOff), response, &responseLen, 0) != IFD_SUCCESS)
 		{
 			DEBUG_CRITICAL("Polling OFF failed");
 		}
@@ -891,7 +1069,7 @@ static int ACR83_GetFirmwareVersion(unsigned int reader_index, unsigned int *pFi
 	unsigned char response[3 + 6];
 	unsigned int responseLen = sizeof(response);
 
-	if (CmdEscape(reader_index, command, commandLen, response, &responseLen) == IFD_SUCCESS)
+	if (CmdEscape(reader_index, command, commandLen, response, &responseLen, 0) == IFD_SUCCESS)
 	{
 		if ((responseLen >= 7) && (response[0] == 0x84))
 		{
@@ -921,7 +1099,7 @@ static int ACR83_DisplayLcdMessage(unsigned int reader_index, const char *messag
 	// Copy message to command
 	memcpy(command + 5, message, messageLen);
 
-	if (CmdEscape(reader_index, command, commandLen, response, &responseLen) == IFD_SUCCESS)
+	if (CmdEscape(reader_index, command, commandLen, response, &responseLen, 0) == IFD_SUCCESS)
 	{
 		if ((responseLen >= 5) && (response[0] == 0x85) &&
 			(response[3] == 0) && (response[4] == 0))
@@ -940,7 +1118,7 @@ static int ACR1222_GetFirmwareVersion(unsigned int reader_index, char *firmwareV
 	unsigned char response[300];
 	unsigned int responseLen = sizeof(response);
 
-	if (CmdEscape(reader_index, command, sizeof(command), response, &responseLen) == IFD_SUCCESS)
+	if (CmdEscape(reader_index, command, sizeof(command), response, &responseLen, 0) == IFD_SUCCESS)
 	{
 		if (*pFirmwareVersionLen >= responseLen - 5 + 1)
 		{
