@@ -1,7 +1,8 @@
 /*
     commands.c: Commands sent to the card
-    Copyright (C) 2003-2004   Ludovic Rousseau
-    Copyright (C) 2010-2012   Advanced Card Systems Ltd.
+    Copyright (C) 2003-2010   Ludovic Rousseau
+    Copyright (C) 2005 Martin Paljak
+    Copyright (C) 2010-2014   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -19,17 +20,28 @@
 */
 
 /*
- * $Id: commands.c 4235 2009-05-29 11:42:46Z rousseau $
+ * $Id: commands.c 6975 2014-09-04 11:33:05Z rousseau $
  */
 
+#include <config.h>
+
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <pcsclite.h>
 #include <ifdhandler.h>
 #include <reader.h>
 
-#include "config.h"
 #include "misc.h"
 #include "commands.h"
 #include "openct/proto-t1.h"
@@ -52,11 +64,18 @@
 #define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
 #endif
 
-// Fix problem using pcsc-lite 1.6.x header files
-#ifndef IFD_ERROR_INSUFFICIENT_BUFFER
-#define IFD_ERROR_INSUFFICIENT_BUFFER 618
+#ifndef BSWAP_16
+#define BSWAP_8(x)  ((x) & 0xff)
+#define BSWAP_16(x) ((BSWAP_8(x) << 8) | BSWAP_8((x) >> 8))
+#define BSWAP_32(x) ((BSWAP_16(x) << 16) | BSWAP_16((x) >> 16))
 #endif
 
+#define CHECK_STATUS(res) \
+	if (STATUS_NO_SUCH_DEVICE == res) \
+		return IFD_NO_SUCH_DEVICE; \
+	if (STATUS_SUCCESS != res) \
+		return IFD_COMMUNICATION_ERROR;
+
 /* internal functions */
 static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
@@ -75,6 +94,7 @@ static RESPONSECODE CmdXfrBlockTPDU_T1(unsigned int reader_index,
 	unsigned char rx_buffer[]);
 
 static void i2dw(int value, unsigned char *buffer);
+static unsigned int bei2i(unsigned char *buffer);
 
 
 /*****************************************************************************
@@ -93,7 +113,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 #ifndef TWIN_SERIAL
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char pcbuffer[SIZE_GET_SLOT_STATUS];
@@ -123,7 +143,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 		return IFD_SUCCESS;
 	}
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char tmp[MAX_ATR_SIZE+1];
@@ -183,19 +203,19 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 
 		if ((1 == voltage) && !(bVoltageSupport & 1))
 		{
-			DEBUG_INFO("5V requested but not support by reader");
+			DEBUG_INFO1("5V requested but not support by reader");
 			voltage = 2;	/* 3V */
 		}
 
 		if ((2 == voltage) && !(bVoltageSupport & 2))
 		{
-			DEBUG_INFO("3V requested but not support by reader");
+			DEBUG_INFO1("3V requested but not support by reader");
 			voltage = 3;	/* 1.8V */
 		}
 
 		if ((3 == voltage) && !(bVoltageSupport & 4))
 		{
-			DEBUG_INFO("1.8V requested but not support by reader");
+			DEBUG_INFO1("1.8V requested but not support by reader");
 			voltage = 0;	/* auto */
 		}
 	}
@@ -209,16 +229,14 @@ again:
 	cmd[8] = cmd[9] = 0; /* RFU */
 
 	res = WritePort(reader_index, sizeof(cmd), cmd);
-	if (res != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(res)
 
 	/* reset available buffer size */
 	/* needed if we go back after a switch to ISO mode */
 	*nlength = length;
 
 	res = ReadPort(reader_index, nlength, buffer);
-	if (res != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(res)
 
 	if (*nlength < STATUS_OFFSET+1)
 	{
@@ -239,7 +257,7 @@ again:
 			unsigned int res_length = sizeof(res_tmp);
 
 			if ((return_value = CmdEscape(reader_index, cmd_tmp,
-				sizeof(cmd_tmp), res_tmp, &res_length)) != IFD_SUCCESS)
+				sizeof(cmd_tmp), res_tmp, &res_length, 0)) != IFD_SUCCESS)
 				return return_value;
 
 			/* avoid looping if we can't switch mode */
@@ -252,7 +270,9 @@ again:
 		/* continue with 3 volts and 5 volts */
 		if (voltage > 1)
 		{
+#ifndef NO_LOG
 			const char *voltage_code[] = { "auto", "5V", "3V", "1.8V" };
+#endif
 
 			DEBUG_INFO3("Power up with %s failed. Try with %s.",
 				voltage_code[voltage], voltage_code[voltage-1]);
@@ -287,10 +307,13 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 {
 	unsigned char cmd[11+14+TxLength];
 	unsigned int a, b;
+	PIN_VERIFY_STRUCTURE *pvs;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int old_read_timeout;
 	RESPONSECODE ret;
+	status_t res;
 
+	pvs = (PIN_VERIFY_STRUCTURE *)TxBuffer;
 	cmd[0] = 0x69;	/* Secure */
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
@@ -305,6 +328,20 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		return IFD_NOT_SUPPORTED;
 	}
 
+	/* On little endian machines we are all set. */
+	/* If on big endian machine and caller is using host byte order */
+	if ((pvs->ulDataLength + 19  == TxLength) &&
+		(bei2i((unsigned char*)(&pvs->ulDataLength)) == pvs->ulDataLength))
+	{
+		DEBUG_INFO1("Reversing order from big to little endian");
+		/* If ulDataLength is big endian, assume others are too */
+		/* reverse the byte order for 3 fields */
+		pvs->wPINMaxExtraDigit = BSWAP_16(pvs->wPINMaxExtraDigit);
+		pvs->wLangId = BSWAP_16(pvs->wLangId);
+		pvs->ulDataLength = BSWAP_32(pvs->ulDataLength);
+	}
+	/* At this point we now have the above 3 variables in little endian */
+
 	if (dw2i(TxBuffer, 15) + 19 != TxLength) /* ulDataLength field coherency */
 	{
 		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 15) + 19, TxLength);
@@ -322,7 +359,8 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 #ifdef BOGUS_PINPAD_FIRMWARE
 	/* bug circumvention for the GemPC Pinpad */
-	if (GEMPCPINPAD == ccid_descriptor->readerID)
+	if ((GEMPCPINPAD == ccid_descriptor->readerID)
+		|| (VEGAALPHA == ccid_descriptor->readerID))
 	{
 		/* the firmware reject the cases: 00h No string and FFh default
 		 * CCID message. The only value supported is 01h (display 1 message) */
@@ -344,7 +382,8 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 	}
 
-	if (DELLSCRK == ccid_descriptor->readerID)
+	if ((DELLSCRK == ccid_descriptor->readerID)
+		|| (DELLSK == ccid_descriptor->readerID))
 	{
 		/* the firmware rejects the cases: 01h-FEh and FFh default
 		 * CCID message. The only value supported is 00h (no message) */
@@ -354,6 +393,21 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 				TxBuffer[8]);
 			TxBuffer[8] = 0x00;
 		}
+
+		/* avoid the command rejection because the Enter key is still
+		 * pressed. Wait a bit for the key to be released */
+		(void)usleep(250*1000);
+	}
+
+	if (DELLSK == ccid_descriptor->readerID)
+	{
+		/* the 2 bytes of wPINMaxExtraDigit are reversed */
+		int tmp;
+
+		tmp = TxBuffer[6];
+		TxBuffer[6] = TxBuffer[5];
+		TxBuffer[5] = tmp;
+		DEBUG_INFO1("Correcting wPINMaxExtraDigit for Dell keyboard");
 	}
 #endif
 
@@ -415,7 +469,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 		/* the SPR532 will append the PIN code without any padding */
 		return_value = CmdEscape(reader_index, cmd_tmp, sizeof(cmd_tmp),
-			res_tmp, &res_length);
+			res_tmp, &res_length, 0);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
@@ -428,13 +482,15 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	i2dw(a - 10, cmd + 1);  /* CCID message length */
 
 	old_read_timeout = ccid_descriptor -> readTimeout;
-	// Change timeout to infinite
-	// ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 	ccid_descriptor -> readTimeout = 0;	// Infinite
 
-	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
+	res = WritePort(reader_index, a, cmd);
+	if (STATUS_SUCCESS != res)
 	{
-		ret = IFD_COMMUNICATION_ERROR;
+		if (STATUS_NO_SUCH_DEVICE == res)
+			ret = IFD_NO_SUCH_DEVICE;
+		else
+			ret = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
@@ -455,8 +511,72 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		}
 		else
 		{
-			/* get only the T=1 data */
 			/* FIXME: manage T=1 error blocks */
+
+			/* defines from openct/proto-t1.c */
+			#define PCB 1
+			#define DATA 3
+			#define T1_S_BLOCK		0xC0
+			#define T1_S_RESPONSE		0x20
+			#define T1_S_TYPE(pcb)		((pcb) & 0x0F)
+			#define T1_S_WTX		0x03
+
+			/* WTX S-block */
+			if ((T1_S_BLOCK | T1_S_WTX) == RxBuffer[PCB])
+			{
+/*
+ * The Swiss health care card sends a WTX request before returning the
+ * SW code. If the reader is in TPDU the driver must manage the request
+ * itself.
+ *
+ * received: 00 C3 01 09 CB
+ * openct/proto-t1.c:432:t1_transceive() S-Block request received
+ * openct/proto-t1.c:489:t1_transceive() CT sent S-block with wtx=9
+ * sending: 00 E3 01 09 EB
+ * openct/proto-t1.c:667:t1_xcv() New timeout at WTX request: 23643 sec
+ * received: 00 40 02 90 00 D2
+*/
+				ct_buf_t tbuf;
+				unsigned char sblk[1]; /* we only need 1 byte of data */
+				t1_state_t *t1 = &get_ccid_slot(reader_index)->t1;
+				unsigned int slen;
+				int oldReadTimeout;
+
+				DEBUG_COMM2("CT sent S-block with wtx=%u", RxBuffer[DATA]);
+				t1->wtx = RxBuffer[DATA];
+
+				oldReadTimeout = ccid_descriptor->readTimeout;
+				if (t1->wtx > 1)
+				{
+					/* set the new temporary timeout at WTX card request */
+					ccid_descriptor->readTimeout *= t1->wtx;
+					DEBUG_INFO2("New timeout at WTX request: %d sec",
+							ccid_descriptor->readTimeout);
+				}
+
+				ct_buf_init(&tbuf, sblk, sizeof(sblk));
+				t1->wtx = RxBuffer[DATA];
+				ct_buf_putc(&tbuf, RxBuffer[DATA]);
+
+				slen = t1_build(t1, RxBuffer, 0,
+					T1_S_BLOCK | T1_S_RESPONSE | T1_S_TYPE(RxBuffer[PCB]),
+					&tbuf, NULL);
+
+				ret = CCID_Transmit(t1 -> lun, slen, RxBuffer, 0, t1->wtx);
+				if (ret != IFD_SUCCESS)
+					return ret;
+
+				/* I guess we have at least 6 bytes in RxBuffer */
+				*RxLength = 6;
+				ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
+				if (ret != IFD_SUCCESS)
+					return ret;
+
+				/* Restore initial timeout */
+				ccid_descriptor->readTimeout = oldReadTimeout;
+			}
+
+			/* get only the T=1 data */
 			memmove(RxBuffer, RxBuffer+3, *RxLength -4);
 			*RxLength -= 4;	/* remove NAD, PCB, LEN and CRC */
 		}
@@ -468,6 +588,44 @@ end:
 } /* SecurePINVerify */
 
 
+#ifdef BOGUS_PINPAD_FIRMWARE
+/*****************************************************************************
+ *
+ *					has_gemalto_modify_pin_bug
+ *
+ ****************************************************************************/
+static int has_gemalto_modify_pin_bug(_ccid_descriptor *ccid_descriptor)
+{
+	/* Bug not present by default */
+	int has_bug = 0;
+
+	/* Covadis Véga-Alpha reader */
+	if (VEGAALPHA == ccid_descriptor->readerID)
+	{
+		/* This reader has the bug (uses a Gemalto firmware) */
+		has_bug = 1;
+	}
+	else
+	{
+		/* Gemalto reader */
+		if ((GET_VENDOR(ccid_descriptor->readerID) == VENDOR_GEMALTO))
+		{
+			has_bug = 1; /* assume it has the bug */
+
+			if (ccid_descriptor->gemalto_firmware_features &&
+				ccid_descriptor->gemalto_firmware_features->bNumberMessageFix)
+			{
+				/* A Gemalto reader has the ModifyPIN structure bug */
+				/* unless it explicitly reports it has been fixed */
+				has_bug = 0;
+			}
+		}
+	}
+
+	return has_bug;
+} /* has_gemalto_modify_pin_bug */
+#endif
+
 /*****************************************************************************
  *
  *					SecurePINModify
@@ -479,13 +637,17 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 {
 	unsigned char cmd[11+19+TxLength];
 	unsigned int a, b;
+	PIN_MODIFY_STRUCTURE *pms;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int old_read_timeout;
 	RESPONSECODE ret;
+	status_t res;
 #ifdef BOGUS_PINPAD_FIRMWARE
-	int bNumberMessages = 0; /* for GemPC Pinpad */
+	int bNumberMessage = 0; /* for GemPC Pinpad */
+	int gemalto_modify_pin_bug;
 #endif
 
+	pms = (PIN_MODIFY_STRUCTURE *)TxBuffer;
 	cmd[0] = 0x69;	/* Secure */
 	cmd[5] = ccid_descriptor->bCurrentSlotIndex;	/* slot number */
 	cmd[6] = (*ccid_descriptor->pbSeq)++;
@@ -500,6 +662,21 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		return IFD_NOT_SUPPORTED;
 	}
 
+	/* On little endian machines we are all set. */
+	/* If on big endian machine and caller is using host byte order */
+	if ((pms->ulDataLength + 24  == TxLength) &&
+		(bei2i((unsigned char*)(&pms->ulDataLength)) == pms->ulDataLength))
+	{
+		DEBUG_INFO1("Reversing order from big to little endian");
+		/* If ulDataLength is big endian, assume others are too */
+		/* reverse the byte order for 3 fields */
+		pms->wPINMaxExtraDigit = BSWAP_16(pms->wPINMaxExtraDigit);
+		pms->wLangId = BSWAP_16(pms->wLangId);
+		pms->ulDataLength = BSWAP_32(pms->ulDataLength);
+	}
+	/* At this point we now have the above 3 variables in little endian */
+
+
 	if (dw2i(TxBuffer, 20) + 24 != TxLength) /* ulDataLength field coherency */
 	{
 		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 20) + 24, TxLength);
@@ -532,22 +709,23 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if ((SPR532 == ccid_descriptor->readerID)
 		|| (CHERRYST2000 == ccid_descriptor->readerID))
 	{
-		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+		TxBuffer[11] = 0x03; /* set bNumberMessage to 3 so that
 								all bMsgIndex123 are filled */
 		TxBuffer[14] = TxBuffer[15] = TxBuffer[16] = 0;	/* bMsgIndex123 */
 	}
 
 	/* the bug is a bit different than for the Cherry ST 2000C
-	 * with bNumberMessages < 3 the command seems to be accepted
+	 * with bNumberMessage < 3 the command seems to be accepted
 	 * and the card sends 6B 80 */
 	if (CHERRYXX44 == ccid_descriptor->readerID)
 	{
-		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+		TxBuffer[11] = 0x03; /* set bNumberMessage to 3 so that
 								all bMsgIndex123 are filled */
 	}
 
 	/* bug circumvention for the GemPC Pinpad */
-	if (GEMPCPINPAD == ccid_descriptor->readerID)
+	if ((GEMPCPINPAD == ccid_descriptor->readerID)
+		|| (VEGAALPHA == ccid_descriptor->readerID))
 	{
 		/* The reader does not support, and actively reject, "max size reached"
 		 * and "timeout occured" validation conditions */
@@ -557,13 +735,21 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 				TxBuffer[10]);
 			TxBuffer[10] = 0x02;	/* validation key pressed */
 		}
+	}
 
-		/* the reader does not support any other value than 3 for the number
-		 * of messages */
-		bNumberMessages = TxBuffer[11];
+	gemalto_modify_pin_bug = has_gemalto_modify_pin_bug(ccid_descriptor);
+	if (gemalto_modify_pin_bug)
+	{
+		DEBUG_INFO1("Gemalto CCID Modify Pin Bug");
+
+		/* The reader requests a value for bMsgIndex2 and bMsgIndex3
+		 * even if they should not be present. So we fake
+		 * bNumberMessage=3.  The real number of messages will be
+		 * corrected later in the code */
+		bNumberMessage = TxBuffer[11];
 		if (0x03 != TxBuffer[11])
 		{
-			DEBUG_INFO2("Correct bNumberMessages for GemPC Pinpad (was %d)",
+			DEBUG_INFO2("Correct bNumberMessage for GemPC Pinpad (was %d)",
 				TxBuffer[11]);
 			TxBuffer[11] = 0x03; /* 3 messages */
 		}
@@ -637,24 +823,26 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if ((SPR532 == ccid_descriptor->readerID)
 		|| (CHERRYST2000 == ccid_descriptor->readerID))
 	{
-		cmd[21] = 0x00; /* set bNumberMessages to 0 */
+		cmd[21] = 0x00; /* set bNumberMessage to 0 */
 	}
 
-	if (GEMPCPINPAD == ccid_descriptor->readerID)
-		cmd[21] = bNumberMessages;	/* restore the real value */
+	if (gemalto_modify_pin_bug)
+		cmd[21] = bNumberMessage;	/* restore the real value */
 #endif
 
 	/* We know the size of the CCID message now */
 	i2dw(a - 10, cmd + 1);	/* command length (includes bPINOperation) */
 
 	old_read_timeout = ccid_descriptor -> readTimeout;
-	// Change timeout to infinite
-	// ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 	ccid_descriptor -> readTimeout = 0;	// Infinite
 
-	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
+	res = WritePort(reader_index, a, cmd);
+	if (STATUS_SUCCESS != res)
 	{
-		ret = IFD_COMMUNICATION_ERROR;
+		if (STATUS_NO_SUCH_DEVICE == res)
+			ret = IFD_NO_SUCH_DEVICE;
+		else
+			ret = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
@@ -695,17 +883,24 @@ end:
  ****************************************************************************/
 RESPONSECODE CmdEscape(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
-	unsigned char RxBuffer[], unsigned int *RxLength)
+	unsigned char RxBuffer[], unsigned int *RxLength, unsigned int timeout)
 {
 	unsigned char *cmd_in, *cmd_out;
 	status_t res;
 	unsigned int length_in, length_out;
 	RESPONSECODE return_value = IFD_SUCCESS;
-	//int old_read_timeout;
+	int old_read_timeout;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
-	//old_read_timeout = ccid_descriptor -> readTimeout;
-	//ccid_descriptor -> readTimeout = 30;	/* 30 seconds */
+	/* a value of 0 do not change the default read timeout */
+	if (timeout > 0)
+	{
+		old_read_timeout = ccid_descriptor -> readTimeout;
+
+		// Set infinite timeout
+		ccid_descriptor -> readTimeout = (timeout == (unsigned int) -1) ?
+			0 : timeout;
+	}
 
 again:
 	/* allocate buffers */
@@ -738,10 +933,15 @@ again:
 	if (res != STATUS_SUCCESS)
 	{
 		free(cmd_out);
-		return_value = IFD_COMMUNICATION_ERROR;
+		if (STATUS_NO_SUCH_DEVICE == res)
+			return_value = IFD_NO_SUCH_DEVICE;
+		else
+			return_value = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
+time_request:
+	length_out = 10 + *RxLength;
 	res = ReadPort(reader_index, &length_out, cmd_out);
 
 	/* replay the command if NAK
@@ -757,7 +957,10 @@ again:
 	if (res != STATUS_SUCCESS)
 	{
 		free(cmd_out);
-		return_value = IFD_COMMUNICATION_ERROR;
+		if (STATUS_NO_SUCH_DEVICE == res)
+			return_value = IFD_NO_SUCH_DEVICE;
+		else
+			return_value = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
@@ -769,6 +972,12 @@ again:
 		goto end;
 	}
 
+	if (cmd_out[STATUS_OFFSET] & CCID_TIME_EXTENSION)
+	{
+		DEBUG_COMM2("Time extension requested: 0x%02X", cmd_out[ERROR_OFFSET]);
+		goto time_request;
+	}
+
 	if (cmd_out[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd_out[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -785,7 +994,9 @@ again:
 	free(cmd_out);
 
 end:
-	//ccid_descriptor -> readTimeout = old_read_timeout;
+	if (timeout > 0)
+		ccid_descriptor -> readTimeout = old_read_timeout;
+
 	return return_value;
 } /* Escape */
 
@@ -804,7 +1015,7 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 #ifndef TWIN_SERIAL
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 
@@ -821,7 +1032,7 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 		return IFD_SUCCESS;
 	}
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char buffer[3];
@@ -857,13 +1068,11 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 	cmd[7] = cmd[8] = cmd[9] = 0; /* RFU */
 
 	res = WritePort(reader_index, sizeof(cmd), cmd);
-	if (res != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(res)
 
 	length = sizeof(cmd);
 	res = ReadPort(reader_index, &length, cmd);
-	if (res != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(res)
 
 	if (length < STATUS_OFFSET+1)
 	{
@@ -895,7 +1104,7 @@ RESPONSECODE CmdGetSlotStatus(unsigned int reader_index, unsigned char buffer[])
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 #ifndef TWIN_SERIAL
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char status[1];
@@ -935,7 +1144,7 @@ again_status:
 		return IFD_SUCCESS;
 	}
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char buffer_tmp[3];
@@ -977,17 +1186,11 @@ again_status:
 	cmd[7] = cmd[8] = cmd[9] = 0; /* RFU */
 
 	res = WritePort(reader_index, sizeof(cmd), cmd);
-	if (res != STATUS_SUCCESS)
-	{
-		if (STATUS_NO_SUCH_DEVICE == res)
-			return IFD_NO_SUCH_DEVICE;
-		return IFD_COMMUNICATION_ERROR;
-	}
+	CHECK_STATUS(res)
 
 	length = SIZE_GET_SLOT_STATUS;
 	res = ReadPort(reader_index, &length, buffer);
-	if (res != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(res)
 
 	if (length < STATUS_OFFSET+1)
 	{
@@ -1067,7 +1270,7 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 				if (protocol == T_1)
 					return_value = CmdXfrBlockTPDU_T1(reader_index, tx_length,
 						tx_buffer, rx_length, rx_buffer);
- 				else
+				else
 					return_value = IFD_PROTOCOL_NOT_SUPPORTED;
 			}
 			break;
@@ -1094,7 +1297,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	status_t ret;
 
 #ifndef TWIN_SERIAL
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 
@@ -1112,7 +1315,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 		return IFD_SUCCESS;
 	}
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 
@@ -1147,10 +1350,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	memcpy(cmd+10, tx_buffer, tx_length);
 
 	ret = WritePort(reader_index, 10+tx_length, cmd);
-	if (STATUS_NO_SUCH_DEVICE == ret)
-		return IFD_NO_SUCH_DEVICE;
-	if (ret != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	CHECK_STATUS(ret)
 
 	return IFD_SUCCESS;
 } /* CCID_Transmit */
@@ -1168,14 +1368,20 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 	unsigned int length;
 	RESPONSECODE return_value = IFD_SUCCESS;
 	status_t ret;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	unsigned int old_timeout;
 
 #ifndef TWIN_SERIAL
-	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
-
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
+		unsigned char pcbuffer[SIZE_GET_SLOT_STATUS];
 		int r;
 
+		/* wait for ready */
+		r = CmdGetSlotStatus(reader_index, pcbuffer);
+		if (r != IFD_SUCCESS)
+			return r;
+
 		/* Data Block */
 		r = ControlUSB(reader_index, 0xA1, 0x6F, 0, rx_buffer, *rx_length);
 
@@ -1192,7 +1398,7 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 		return IFD_SUCCESS;
 	}
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
 		unsigned char rx_tmp[4];
@@ -1283,15 +1489,16 @@ time_request_ICCD_B:
 	}
 #endif
 
+	/* store the original value of read timeout*/
+	old_timeout = ccid_descriptor -> readTimeout;
+
 time_request:
 	length = sizeof(cmd);
 	ret = ReadPort(reader_index, &length, cmd);
-	if (ret != STATUS_SUCCESS)
-	{
-		if (STATUS_NO_SUCH_DEVICE == ret)
-			return IFD_NO_SUCH_DEVICE;
-		return IFD_COMMUNICATION_ERROR;
-	}
+
+	/* restore the original value of read timeout */
+	ccid_descriptor -> readTimeout = old_timeout;
+	CHECK_STATUS(ret)
 
 	if (length < STATUS_OFFSET+1)
 	{
@@ -1341,6 +1548,12 @@ time_request:
 	if (cmd[STATUS_OFFSET] & CCID_TIME_EXTENSION)
 	{
 		DEBUG_COMM2("Time extension requested: 0x%02X", cmd[ERROR_OFFSET]);
+
+		/* compute the new value of read timeout */
+		if (cmd[ERROR_OFFSET] > 0)
+			ccid_descriptor -> readTimeout *= cmd[ERROR_OFFSET];
+
+		DEBUG_COMM2("New timeout: %d ms", ccid_descriptor -> readTimeout);
 		goto time_request;
 	}
 
@@ -1396,7 +1609,7 @@ static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
 	unsigned int local_rx_length = 0, received_length;
 	int buffer_overflow = 0;
 
-	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
 	{
 		/* length is on 16-bits only
 		 * if a size > 0x1000 is used then usb_control_msg() fails with
@@ -1622,14 +1835,15 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 	unsigned int proc_len, int is_rcv)
 {
 	RESPONSECODE return_value;
-	unsigned int remain_len;
-	unsigned char tmp_buf[512];
 	unsigned int ret_len;
 
 	DEBUG_COMM2("Enter, is_rcv = %d", is_rcv);
 
 	if (is_rcv == 1)
 	{	/* Receiving mode */
+		unsigned int remain_len;
+		unsigned char tmp_buf[512];
+
 		if (*in_len > 0)
 		{	/* There are still available data in our buffer */
 			if (*in_len >= proc_len)
@@ -1795,7 +2009,7 @@ static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
 
 	DEBUG_COMM2("T=0: %d bytes", snd_len);
 
-	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	if (PROTOCOL_ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		unsigned char pcbuffer[SIZE_GET_SLOT_STATUS];
 		unsigned int backup_len;
@@ -1829,6 +2043,7 @@ static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
 			return return_value;
 
 		/* wait for ready */
+		pcbuffer[0] = 0;
 		return_value = CmdGetSlotStatus(reader_index, pcbuffer);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
@@ -2031,6 +2246,7 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 {
 	unsigned char cmd[10+length];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	status_t res;
 
 	DEBUG_COMM2("length: %d bytes", length);
 
@@ -2043,12 +2259,12 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 
 	memcpy(cmd+10, buffer, length);
 
-	if (WritePort(reader_index, 10+length, cmd) != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	res = WritePort(reader_index, 10+length, cmd);
+	CHECK_STATUS(res)
 
 	length = sizeof(cmd);
-	if (ReadPort(reader_index, &length, cmd) != STATUS_SUCCESS)
-		return IFD_COMMUNICATION_ERROR;
+	res = ReadPort(reader_index, &length, cmd);
+	CHECK_STATUS(res)
 
 	if (length < STATUS_OFFSET+1)
 	{
@@ -2097,3 +2313,13 @@ static void i2dw(int value, unsigned char buffer[])
 	buffer[3] = (value >> 24) & 0xFF;
 } /* i2dw */
 
+/*****************************************************************************
+*
+*                  bei2i (big endian integer to host order interger)
+*
+****************************************************************************/
+
+static unsigned int bei2i(unsigned char buffer[])
+{
+	return (buffer[0]<<24) + (buffer[1]<<16) + (buffer[2]<<8) + buffer[3];
+}
